;**************************************************************************
;****	File : Ex4.asm
;****	Read VR1 with the analog voltage input and convert the voltage
;****   to 10-bit result, Conver the result to ASCII code then send to
;****   VT-100 Terminal. 
;****   Press "c" to enable sending A/D result to Terminal
;****   Press "p" to stop sending A/D result to Terminal
;**************************************************************************

			list 	p = 18F452
			include <p18F452.inc>
			
;
			CBLOCK	0x00
			C_Hold_Delay			; variable used for the sample hold charge time
			TxD_Flag				; 500mS time flag, Send TxD data to Terminal
			Hex_Temp
			ENDC	

			CBLOCK	0x080
			WREG_TEMP				;variable used for context saving 
			STATUS_TEMP				;variable used for context saving
			BSR_TEMP				;variable used for context saving
			ENDC
;
#define		TMR1_VAL	.32768		; Define Timer1 time period are 1 SEC
;					       		
;********************************************************************
;****	RESET Vector @ 0x0000
;********************************************************************

			org		0x0000 			; Reset Vector 
 			bra		Init
;
			org		0x0008			; High Priority Interrupt Vector
			bra		Hi_ISRs
;
			org		0x0018			; Low Priority Interrupt Vector
			bra		Low_ISRs
;
;********************************************************************
;****	The Main Program start from Here !! 
;********************************************************************

			org		0x0020
Init:
			call	Init_IO
			call	Init_Timer1
			call	Init_AD
			call	Init_USART
;
			bsf		RCON,IPEN		; Enable Interrupt priority
			bsf		INTCON,GIEH		; Enable High Priority Interrupt for Timer1
			bsf		INTCON,GIEL		; Enable Low Priority Interrupt for RxD of USART
			clrf	TxD_Flag		; Clear the 500ms time out flag
;
Main:	
			nop
			btfsc	TxD_Flag,1		; if = 0, continue sending A/D result to Terminal
			bra		Main			; if = 1, Received "p" from terminal, Now is "pause"
;
			btfss	TxD_Flag,0		; Check time period is 1 Sec?
			bra 	Main
			bcf		TxD_Flag,0		; Clear 1 SEC time out flat
			call	AD_Convert
;
 			movf	ADRESH,W		; Conver A/D <b9:b8> to ASCII and send to USART
 			andlw	b'00000011'
 			call	Hex_ASCII
 			call 	Tx_a_Byte
;
 			swapf	ADRESL,W		; Conver A/D <b7:b4> to ASCII and send to USART
 			andlw	h'0F'
 			call	Hex_ASCII
 			call	Tx_a_Byte
;
 			movf	ADRESL,W		; Conver A/D <b3:b0> to ASCII and send to USART
 			andlw	h'0F'
 			call	Hex_ASCII
 			call	Tx_a_Byte
;
			movlw	h'0a'			; Send 0x0A & 0x0D to terminal
			call 	Tx_a_Byte
			movlw	h'0d'
			call 	Tx_a_Byte
;
			bra		Main
			
			
;
;******   Send a byte to USART   ******
Tx_a_Byte:
			?????							; Send a byte vis USART
			nop								; 
			btfss	PIR1,TXIF				; Does the data sent out ?
			bra		$-4						; No, Loop check the TXIF
			?????							; Yes, completed to send the date, clear the TXIF
			return
;
;******* Convert low nibble to ASCII code   *******
Hex_ASCII:
 			andlw	h'0F'					; Make sure high nibble are "0000"
 			movwf	Hex_Temp
 			movlw	h'9'					; Compare with 9
 			cpfsgt  Hex_Temp
 			bra		Less_9
 			movf	Hex_Temp,W				; > 9, the number plus 0x37
 			addlw	h'37'
 			return
Less_9		movf	Hex_Temp,W				; < = 9, the number plus 0x30
 			addlw	h'30'
 			return
 			
;
;*****    Connvert the 10-bit A/D   *****
AD_Convert:
			call	C_Hold_Time				; Delay 50uS for the sample hold time
			bsf		ADCON0,GO				; Start convert the A/D
			nop								; Nop
			btfsc	ADCON0,GO				; Check the A/D completed ?
			bra		$-4						; A/D isn't completed yet, continut check.
			return 
;
;***********************************************************************
;****		Initial I/O Port 
;***********************************************************************
Init_IO:
			clrf	TRISD					; Set PORTD for output port
			clrf	PORTD
   			bsf		TRISA,RA0				; Set RA0 with input mode
			return
;
;***********************************************************************
;****		Initial Timer1 as a 1 Sec Timer 
;***********************************************************************
Init_Timer1:
			movlw	B'10001111'				; 16-bit mode , Prescaler = 1:1 , Async mode
			movwf	T1CON					; Use external 32768Hz crystal , Timer1 On
;
	
			movlw	(.65536-TMR1_VAL)/.256	; Calcute the MSB value for the Timer1
			movwf	TMR1H
			movlw	(.65536-TMR1_VAL)%.256	; Calcute the LSB value for the Timer1
			movwf	TMR1L
;
			bsf		IPR1,TMR1IP				; Set Timer1 for high priority interrupt
			bcf		PIR1,TMR1IF				; Clear Timer1 Interrupt flag
			bsf		PIE1,TMR1IE				; Enable Timer1 Interrupt
;
			return
;
;***********************************************************************
;****		Initial A/D converter 
;***********************************************************************
Init_AD:
			movlw	b'10001110'				; Right Justified, Fosc/32
			movwf	ADCON1					; Set AN0 for the Analog Input
;
  			movlw	b'10000001'				; Fosc/32 , Select AN0 channel
  			movwf	ADCON0					; A/D on
;
			bcf		PIE1,ADIE				; Disable interrupt of A/D

			return
;
;***********************************************************************
;****		Initial USART as 9600,N,8,1 
;***********************************************************************
Init_USART:
			?????							; 8-bit data mode , ASYNC 
			?????							; Low Speed mode, Enable TxD
;
			?????							; Enable Serial Port, 8-bit receive
			?????							; Continuous Receive, Disable Address Detection
;
			?????							; Set baud rate at 9600 with Low Speed mode
			?????
;
			bcf		PIR1,TXIF				; Clear TxD interrupt flag
			bcf		PIE1,TXIE				; Disable TxD interrupt
;
			bcf		IPR1,RCIP	  			; Set the RxD for Low priority interrupt
			bcf		PIR1,RCIF				; Clear RxD interrupt flag
			bsf		PIE1,RCIE				; Enable RxD interrupt
;
			return

;
;***********************************************************************
;****		Sample Hold (Charge) time delay routine (50uS) 
;***********************************************************************
C_Hold_Time:
   			movlw	.50
   			movwf	C_Hold_Delay
   			nop
   			decfsz	C_Hold_Delay,F
   			bra		$-4
   			return

;***************************************************************************************
;****		Hi_ISRs : Hi-Priority Interrupt reotine 
;****
;***************************************************************************************
Hi_ISRs

			bcf		PIR1,TMR1IF				; Clear Timer1 Interrupt flag
;							   		
			movlw	(.65536-TMR1_VAL)/.256	; Calcute the MSB value for the Timer1
			movwf	TMR1H
			movlw	(.65536-TMR1_VAL)%.256	; Calcute the LSB value for the Timer1
			movwf	TMR1L
;
			bsf		TxD_Flag,0				; Set 1 Sec time-out flag
;
			retfie	FAST				 	; Return with shadow register
; 
;***************************************************************************************
;****		Low_ISRs : Low-Priority Interrupt reotine 
;****
;***************************************************************************************
Low_ISRs:
			movff	STATUS,STATUS_TEMP		; save STATUS register
			movff	WREG,WREG_TEMP			; save working register
			movff	BSR,BSR_TEMP			; save BSR register
;
			bcf		PIR1,RCIF				; Clear RxD interrupt flag
			movff	RCREG,PORTD				; Display the Rx data to LED port
;
			movlw	a'c'					; Check the received byte is "c" ?
			cpfseq  RCREG
			bra		No_EQU_C
			bcf		TxD_Flag,1				; Yes, enable A/D transmitted
			clrf	RCREG					; Clear the receive register
			bra		Exit_Low_ISR
No_EQU_C	movlw	a'p'					; Check the received byte is "p" ?
			cpfseq  RCREG
			bra		Exit_Low_ISR			; No, not action
			bsf		TxD_Flag,1				; Yes, disable the A/D transmitted
			clrf	RCREG	
;
Exit_Low_ISR
			movff	BSR_TEMP,BSR			; restore BSR register
			movff	WREG_TEMP,WREG			; restore working register
			movff	STATUS_TEMP,STATUS		; restore STATUS register
			retfie		
; 
	 	  	END					 
